/**************************************************************************/
/* Btrieve Technologies, Inc.                                             */
/* Copyright (c) 1990-1994 Btrieve Technologies, Inc.                     */
/*                                                                        */
/* WDDLSVCS.H                                                             */
/*                                                                        */
/* Structure definitions and function prototypes for using WDDLSVCS.DLL   */
/*                                                                        */
/**************************************************************************/

/* Defined Constants     */

#define PATHLEN  65                /* maximum path length = 64 bytes       */
#define TABLELEN 21
#define FIELDLEN 21                /* maximum field/table name length = 20 */
#define MAXKEYS  24
#define USERLEN  30

#define DICTIONARY_OPEN       3001  /* dictionary has already been opened  */
#define NO_FIELDS_ERR         3002
#define ACCESS_DENIED         3003  /* access to the dictionary denied - a */
                                    /* valid username and password is reqd.*/
#define FIELD_DEF_ERR         3004  /* A field definition during a create  */
                                    /* table is invalid                    */

/* Structure Definitions */

/*----------------------------------------------------------------------------
   TABLEPROPERTIES

   This structure defines the properties of a dictionary table.  It is
   used in both DDLGetTableProperties() and DDLCreateTable() functions.

   The structure is filled with all relevant properties of the table
   including lists of index and field descriptions.

   The index and field lists are allocated as global memory objects and
   are not locked before and after the calls.

   On calls to DDLCreateTable(), the application fills the structure with
   all neccessary information to create the table.

   On calls to DDLGetTableProperties(), the information is returned.

   Any alternate collating sequence is passed as an unlocked global memory
   object as well.
*/
typedef struct tableproperties
{
   BYTE   szTableName [TABLELEN];/* dictionary TableName               */
   BYTE   szPathName [PATHLEN];  /* actual [path\name]of data file     */
   WORD   iRecSize;              /* fixed record length                */
   BOOL   bVariable;             /* variable length records flag       */
   DWORD  dMaxRecSize;           /* fixed + max variable size          */
   BOOL   bWriteAccess;          /* set if user has RW access to table */
   WORD   iIndexCount;           /* count of items in index list       */
   HANDLE hIndexList;            /* index list handle                  */
   WORD   iFieldCount;           /* count of items in field list       */
   HANDLE hFieldList;            /* field list handle                  */
   HANDLE hAltSeq;               /* alt. collating sequence handle     */
} TABLEPROPERTIES;



/*----------------------------------------------------------------------------
   TABLELIST

   This structure lists the tables defined in a dictionary.
*/
typedef struct tablelist
{
   BYTE   szName[TABLELEN];
} TABLELIST;




/*----------------------------------------------------------------------------
   FIELDMAP

   This structure describes the properties of a specific field in a
   table.
*/
typedef struct fieldmap
{
   BYTE   szFieldName [FIELDLEN]; /* field name                        */
   WORD   iType;                  /* data type                         */
   WORD   iOffset;                /* field offset                      */
   WORD   iLength;                /* field length                      */
   WORD   iDecimal;               /* decimal places                    */
   BYTE   bAccessRights;          /* user's access rights on the field */
} FIELDMAP;



/*----------------------------------------------------------------------------
   INDEXMAP

   This structure describes the various indexes associated with a table.
*/
typedef struct indexmap
{
   WORD   iFieldIndex;            /* index into associated field       */
   BYTE   szIndexName[FIELDLEN];  /* index name                        */
   WORD   iIndex;                 /* Btrieve index number              */
   WORD   iSegment;               /* segment index number              */
   WORD   iFlags;                 /* index attributes                  */
} INDEXMAP;



/*----------------------------------------------------------------------------
   ADDINDEXMAP

   This structure describes the various indexes to be added to a table.
   The szFieldName field is a NULL terminated string representing the
   associated field name.

   The application should first successfully create a new index in the actual
   data file before this API is called to add the index to the dictionary.

   If adding a named index, the index name must be specified in the 0 segment
   of the index.
*/
typedef struct addindexmap
{
   BYTE   szFieldName[FIELDLEN];  /* Fieldname associated with the ndx */
   BYTE   szIndexName[FIELDLEN];  /* index name                        */
   WORD   iIndex;                 /* Btrieve index number              */
   WORD   iSegment;               /* segment index number              */
   WORD   iFlags;                 /* index attributes                  */
} ADDINDEXMAP;



/*----------------------------------------------------------------------------
   DDL_VERSION_INFO

   This structure provides version information for both the DDLSVCS module and
   the currently active Btrieve module(s).  If Btrieve requester is active,
   it is signified by the type 'N', and may also have the local client Btrieve
   active.  In this case, the client Btrieve's version information would be
   in the second group of Btrieve versions.  If only the client Btrieve is
   active, it's version inofrmation is in the first group and it's type is
   given as 'W' for Windows, or 'P' for OS/2.
*/

typedef struct versioninfo
{
   WORD       iDDLVersion;
   WORD       iDDLRevision;
   WORD       iDDLType;
   WORD       iBVersion1;
   WORD       iBRevision1;
   WORD       iBType1;
   WORD       iBVersion2;
   WORD       iBRevision2;
   WORD       iBType2;
} DDL_VERSION_INFO;


/* Function Prototypes  */

/*---------------------------------------------------------------------------
    Open dictionary files FILE.DDF, FIELD.DDF, and INDEX.DDF in the
    given directory.  If security is defined for the dictionary, open
    the security dictionary files USER.DDF and RIGHTS.DDF as well.
    The dictionary handle returned is be used with all subsequent
    operations on the dictionary.

    DDLOpenDataDictionary (szPathName,
                           piDictionaryHandle,
                           szUserName,
                           szPassWord)
*/
extern WORD far pascal DDLOpenDataDictionary (LPSTR,
                                              WORD far *,
                                              LPSTR,
                                              LPSTR);


/*---------------------------------------------------------------------------
    Create dictionary files FILE.DDF, FIELD.DDF, and INDEX.DDF in the
    given directory.  The handle to the dictionary is returned.  An
    alternate collating can be specified as a resource handle.  The
    resource should be unlocked.  For specifications on an alternate
    collating sequence refer to the Btrieve manual.

    DDLCreateDataDictionary (szPathName,
                             piDictionaryHandle,
                             hAltSeq)
*/
extern WORD far pascal DDLCreateDataDictionary (LPSTR,
                                                WORD far *,
                                                HANDLE);


/*---------------------------------------------------------------------------
    Close files associated with the dictionary handle

    DDLCloseDataDictionary (iDictionaryHandle)
*/
extern WORD far pascal DDLCloseDataDictionary (WORD);



/*----------------------------------------------------------------------------
    Get the list of tables defined in the specified dictionary

    DDLGetTableNames (iDictionaryHandle, piTableCount, pTableList)
*/
extern WORD far pascal DDLGetTableNames (WORD, WORD far *, HANDLE far *);



/*----------------------------------------------------------------------------
    Get information for the specified table in the dictionary

    DDLGetTableProperties (iDictionaryHandle,
                           pTableProps)
*/
extern WORD far pascal DDLGetTableProperties (WORD,
                                              TABLEPROPERTIES far *);



/*----------------------------------------------------------------------------
    Create a table in the specified dictionary given the TABLEPROPERTIES

    DDLCreateTable (piDictionaryHandle,
                    pTableProperties)
*/
extern WORD far pascal DDLCreateTable (WORD,
                                       TABLEPROPERTIES far *);



/*----------------------------------------------------------------------------
    Drop a table in the specified dictionary

    DDLDropTable (iDictionaryHandle,
                  szTableName)
*/
extern WORD far pascal DDLDropTable (WORD, LPSTR);


/*----------------------------------------------------------------------------
    Add a new index to the existing table specifications for a specified
    table. Multiple index definitions may be added with a single API call.
    The iIndexCount parameter specifies the number of index defintions in the
    list.

    The application should first successfully create a new index in the actual
    data file before this API is called to add the index to the dictionary.

    Index names must be specified in the 0 segment of the index definition.
    If the index is not a named index, set the szIndexName to NULL;

    DDLAddIndex (iDictionaryHandle,
                 szTableName,
                 iIndexCount,
                 pIndexList)
*/
extern WORD far pascal DDLAddIndex (WORD, LPSTR, WORD, ADDINDEXMAP far *);


/*----------------------------------------------------------------------------
    Drop an index in an existing table in the specified dictionary.  The
    reorderIndexes flag applies to Btrieve v6.0 and later.  If the indexes
    are reordered, the flag is returned as TRUE; otherwise, it is FALSE.
    If the applications initializes the flag to FALSE, no reordering will be
    done.

    As indexes may be given a name, the index name may be used rather than
    the index number.  In this case the input index number is ignored.  If the
    index to be dropped is not a named index, pass the szIndexName as NULL.

    DDLDropIndex (iDictionaryHandle,
                  szTableName,
                  szIndexName,
                  iIndex,
                  piReorderIndexes)
*/
extern WORD far pascal DDLDropIndex   (WORD, LPSTR, LPSTR, WORD, BOOL far *);



/*----------------------------------------------------------------------------
     Get version information for the DDLSVCS and Btrieve modules.  The Btrieve
     module(s) may be either requester, client, or both.  If both requester
     and client are active, the requester will be the first version in the
     structure and signified by a type of 'N'.

     DDLGetVersionInfo (pVersionInfo)
*/

extern WORD far pascal DDLGetVersionInfo (DDL_VERSION_INFO far *);
